<?php 

namespace Schaffen\SerasaRemessa\Http\Advanced\PF;
use Carbon\Carbon;


trait Translator {
    public $genders = [
        'F' => 'Feminino',
        'M' => 'Masculino',
    ];

    public $situations = [
        '2' => "REGULAR",
        '3' => "PENDENTE DE REGULARIZAÇÃO",
        '6' => "SUSPENSA",
        '7' => "TITULAR TITULAR FALECIDO",
        '9' => "CANCELADA",
        '4' => "NULA",
        '0' => "CPF NÃO CONFIRMADO NO CADASTRO ATÉ ESTA DATA"
    ];

    public $pendencyTypes = [
        '01' => "Pendências Comerciais (PEFIN)",
        '03' => "Pendências Internas",
        '04' => "Pendências Bancárias (REFIN)"
    ];

    public function getIdentification($blocks) {
        $b001 = $blocks["B001"]->first();
        $b002 = $blocks["B002"]->first();

        return [
            'name' => trim($b001['spelling']),
            'cpf' => $b001['cpf'],
            'birthdate' => Carbon::createFromFormat('Ymd', $b001['birthdate'])->format('Y-m-d'),
            'mothers_name' => trim($b002['mothers_name']),
            'gender' => isset($this->genders[$b002['gender']]) ? $this->genders[$b002['gender']] : null
        ];
    }

    public function getSituation($blocks) {
        $b001 = $blocks["B001"]->first();
        return [
            'description' => $this->situations[$b001['new_situation']],
            'date' => Carbon::createFromFormat('Ymd', $b001['current_date'])->format('Y-m-d'),
        ];
    }

    public function getAddress($blocks) {
        $block = $blocks["B004"]->first();
        return [
            'street' => trim($block['street']),
            'number' => trim($block['number']),
            'complement' => trim($block['complement']),
            'neighborhood' => trim($block['neighborhood']),
            'city' => trim($block['city']),
            'state' => trim($block['state']),
            'cep' => trim($block['cep']),
        ];
    }

    public function getContact($blocks) {
        return [
            'ddd_res' => trim($blocks["B003"][0]['ddd_res']),
            'phone_res' => trim($blocks["B003"][0]['phone_res']),
            'ddd_com' => trim($blocks["B003"][0]['ddd_com']),
            'phone_com' => trim($blocks["B003"][0]['phone_com']),
            'branch' => trim($blocks["B003"][0]['branch']),
            'ddd_cell' => trim($blocks["B003"][0]['ddd_cell']),
            'cell' => trim($blocks["B003"][0]['cell']),
        ];
    }

    public function getSummaries($blocks) {
        $summaries = $this->getPendenciesSummary($blocks);

        $summaries[] = $this->getProtestsSummary($blocks);
        $summaries[] = $this->getOverdueDebtSummary($blocks);
        $summaries[] = $this->getBadCheckSummary($blocks);
        $summaries[] = $this->getJudicialActionsSummary($blocks);
        $summaries[] = $this->getBankruptciesSummary($blocks);

        return array_filter($summaries);
    }

    public function getProtestsSummary($blocks) {
        if(!isset($blocks['B361'])) {
            return null;
        }

        $block = $blocks['B361']->first();
        return [
            'description' => trim($block['desc']),
            'amount' => (int) trim($block['total_amount']),
            'value' => (float) trim($block['value']),
            'smallest_date' => Carbon::createFromFormat("Ym", $block['smallest_date'])->format('Y-m-d'),
            'largest_date' => Carbon::createFromFormat("Ym", $block['largest_date'])->format('Y-m-d'),
            'source' => trim($block['city']) . ' - ' . trim($block['state']),
        ];
    }

    public function getPendenciesSummary($blocks) {
        if(!isset($blocks['B357'])) {
            return null;
        }

        return $blocks['B357']->map(function($block) {
            return [
                'description' => $this->pendencyTypes[trim($block['type_pefin'])],
                'amount' => (int) trim($block['total_amount']),
                'value' => (float) trim($block['total_value']),
                'smallest_date' => Carbon::createFromFormat("Ym", $block['smallest_date'])->format('Y-m-d'),
                'largest_date' => Carbon::createFromFormat("Ym", $block['largest_date'])->format('Y-m-d'),
                'source' => trim($block['source'])
            ];
        })->toArray();
    }

    public function getOverdueDebtSummary($blocks) {
        if(!isset($blocks['B367'])) {
            return null;
        }

        $block = $blocks['B367']->first();

        return [
            'description' => trim($block['description']),
            'amount' => (int) trim($block['total_amount']),
            'value' => (float) trim($block['total_value']),
            'smallest_date' => Carbon::createFromFormat("Ym", $block['smallest_date'])->format('Y-m-d'),
            'largest_date' => Carbon::createFromFormat("Ym", $block['largest_date'])->format('Y-m-d'),
            'source' => trim($block['source'])
        ];
    }

    public function getBadCheckSummary($blocks) {
        if(!isset($blocks['B359'])) {
            return null;
        }

        return $blocks['B359']->map(function($block) {
            return [
                'description' => trim($block['desc']),
                'amount' => (int) trim($block['total_amount']),
                'value' => (float) trim($block['total_value']),
                'smallest_date' => Carbon::createFromFormat("Ym", $block['smallest_date'])->format('Y-m-d'),
                'largest_date' => Carbon::createFromFormat("Ym", $block['largest_date'])->format('Y-m-d'),
                'source' => trim($block['source'])
            ];
        });
    }

    public function getJudicialActionsSummary($blocks) {
        if(!isset($blocks['B363'])) {
            return null;
        }

        return $blocks['B363']->map(function($block) {
            return [
                'description' => trim($block['description']),
                'amount' => (int) trim($block['total_amount']),
                'value' => (float) trim($block['total_value']),
                'smallest_date' => Carbon::createFromFormat("Ym", $block['smallest_date'])->format('Y-m-d'),
                'largest_date' => Carbon::createFromFormat("Ym", $block['largest_date'])->format('Y-m-d'),
                'source' => trim($block['state'])
            ];
        });
    }

    public function getBankruptciesSummary($blocks) {
        if(!isset($blocks['B365'])) {
            return null;
        }

        return $blocks['B365']->map(function($block) {
            return [
                'description' => trim($block['description']),
                'total_amount' => (int) trim($block['total_occur']),
                'smallest_date' => Carbon::createFromFormat("Ym", $block['smallest_date'])->format('Y-m-d'),
                'largest_date' => Carbon::createFromFormat("Ym", $block['largest_date'])->format('Y-m-d'),
                'company' => trim($block['company'])
            ];
        });
    }
    public function getScore($blocks) {
        if(!isset($blocks['B280'])) {
            return null;
        }

        $block = $blocks['B280']->first();

        return [
            'message' => trim($block['message']),
            'score' => (strlen(trim($block['score'])) > 0) ? ((float) trim($block['score'])) : null,
            'rate' => (strlen(trim($block['rate'])) > 0) ? ((float) trim($block['score'])) : null,
            'range' => (strlen(trim($block['range'])) > 0) ? ((float) trim($block['score'])) : null,
        ];
    }

    public function getProtests($blocks) {
        if(!isset($blocks['B362'])) {
            return null;
        }

        return $blocks['B362']->map(function($block) {
            return [
                'date' => Carbon::createFromFormat("Ymd", $block['dt_occur'])->format('Y-m-d'),
                'value' => (float) trim($block['value']),
                'registry' => trim($block['registry']),
                'city' => trim($block['city']),
                'state' => trim($block['state'])
            ];
        });
    }

    public function getPendencies($blocks) {
        if(!isset($blocks['B358'])) {
            return [];
        }

        return $blocks['B358']->map(function($block) {
            return [
                'type_pefin' => trim($block['type_pefin']),
                'contract' => trim($block['contract']),
                'description' => $this->pendencyTypes[trim($block['type_pefin'])],
                'modality' => trim($block['modality']),
                'source' => trim($block['source']),
                'main' => trim($block['main']),
                'date' => Carbon::createFromFormat("Ymd", $block['date_occour'])->format('Y-m-d'),
                'value' => (float) trim($block['value']),
            ];
        })->groupBy('description');
    }

    // Dividas venciadas
    public function getOverdueDebt($blocks) {
        if(!isset($blocks['B368'])) {
            return [];
        }

        return $blocks['B368']->map(function($block) {
            return [
                'contract' => trim($block['title']),
                'modality' => trim($block['nature']),
                'source' => trim($block['source']),
                'date' => Carbon::createFromFormat("Ymd", $block['dt_occur'])->format('Y-m-d'),
                'value' => (float) trim($block['value']),
                'local' => trim($block['local']),
            ];
        });
    }

    // Cheques sem fundo
    public function getBadChecks($blocks) {
        if(!isset($blocks['B360'])) {
            return [];
        }

        return $blocks['B360']->map(function($block) {
            return [
                'number' => trim($block['number_check']),
                'bank' => trim($block['bank']),
                'agency' => trim($block['agency']),
                'date' => Carbon::createFromFormat("Ymd", $block['date_occur'])->format('Y-m-d'),
                'value' => (float) trim($block['value']),
                'city' => trim($block['city']),
                'state' => trim($block['state']),
            ];
        });
    }

    // Ações Judiciais
    public function getJudicialActions($blocks) {
        if(!isset($blocks['B364'])) {
            return [];
        }

        return $blocks['B364']->map(function($block) {
            return [
                'court' => trim($block['court']),
                'nature' => trim($block['nature']),
                'main' => trim($block['main']),
                'date' => Carbon::createFromFormat("Ymd", $block['dt_occur'])->format('Y-m-d'),
                'value' => (float) trim($block['value']),
                'city' => trim($block['city']),
                'state' => trim($block['state']),
            ];
        });
    }

    // Falências
    public function getBankruptcies($blocks) {
        if(!isset($blocks['B366'])) {
            return [];
        }

        return $blocks['B366']->map(function($block) {
            return [
                'court' => trim($block['court']),
                'type_occur' => trim($block['type_occur']),
                'cnpj' => trim($block['cnpj']),
                'company' => trim($block['company']),
                'date' => Carbon::createFromFormat("Ymd", $block['date_ocurr'])->format('Y-m-d'),
                'value' => (float) trim($block['value']),
                'qualif' => trim($block['qualif'])
            ];
        });
    }



    public function getQueries($blocks) {
        if(!isset($blocks['B354'])) {
            return null;
        }

        return $blocks['B354']->map(function($block) {
            return [
                'date' => Carbon::createFromFormat("Ymd", $block['date_occur'])->format('Y-m-d'),
                'source' => trim($block['source']),
            ];
        });
    }

    public function getQueriesSummary($blocks) {
        if(!isset($blocks['B353'])) {
            return null;
        }

        $block = $blocks['B353']->first();

        $currentDate = Carbon::createFromFormat("Ym", $block['credit_current_date']);

        return [
            'credit' => [
                'total' => (int) trim($block['credit_total']),
                'current' => (int) trim($block['credit_current']),
                'month_1' => (int) trim($block['credit_month_1']),
                'month_2' => (int) trim($block['credit_month_2']),
                'month_3' => (int) trim($block['credit_month_3']),
            ],

            'check' => [
                'total' => (int) trim($block['check_total']),
                'current' => (int) trim($block['check_current']),
                'month_1' => (int) trim($block['check_month_1']),
                'month_2' => (int) trim($block['check_month_2']),
                'month_3' => (int) trim($block['check_month_3']),
            ],

            'months' => [
                'current' => $currentDate->format('Y-m-d'),
                'month_1' => $currentDate->subMonth(1)->format('Y-m-d'),
                'month_2' => $currentDate->subMonth(2)->format('Y-m-d'),
                'month_3' => $currentDate->subMonth(3)->format('Y-m-d'),
            ],
        ];
    }

    public function getPartnerParticipation($blocks) {
        if(!isset($blocks['B352'])) {
            return null;
        }

        $b35P = $blocks['B35P'];

        return $blocks['B352']->map(function($block, $key) use($b35P) {
            return [
                'enterprise' => trim($block['enterprise']),
                'cnpj' => trim($block['cnpj']),
                'perc_partic' => (float) trim($block['perc_partic']) / 10,
                'state' => trim($block['state']),
                'status' => trim($block['status']),
                'date_start' => trim($block['date_start']),
                'date_last_update' => trim($block['date_last_update']),
                'restriction' => isset($b35P[$key]) && $b35P[$key] == 'S' ? "Sim" : "Não"
            ];
        });
    }

    public function getSyntheticRegistration($blocks) {
        if(!isset($blocks['B005']) || !isset($blocks['B006'])) {
            return null;
        }

        
        $blockB005 = $blocks['B005']->first();
        $blockB006 = $blocks['B006']->first();

        return [
            'occupation' => trim($blockB005['occupation']),
            'income' => (float) trim($blockB005['income']),
            'perc_partic' => (float) trim($blockB005['perc_partic']),
            
            'enterprise' => trim($blockB006['enterprise']),
            'since' => (float) trim($blockB006['since']),
            'profession' => (float) trim($blockB006['profession']),
            'role' => (float) trim($blockB006['role']),
        ];
    }

    public function getAdditionalData($blocks) {
        if(!isset($blocks['F900'])) {
            return null;
        }

        $f900 = $blocks['F900']->groupBy('name');

        return [
            $this->getEstimatedIncome($f900),
            $this->getPaymentCapacity($f900),
            $this->getIncomeCommitment($f900)
        ];

    }

    public function getEstimatedIncome($blocks) {
        if(!isset($blocks['estimated_income'])) {
            return null;
        }

        $block = $blocks['estimated_income']->first();

        if(!$block || $block['sub_type'] == '0') {
            return [
                'description' => 'Renda Estimada',
                'message' => trim($block['message']),
                'value' => null
            ];
        }

        return [
            'description' => 'Renda Estimada',
            'value' => ((float) $block['income']) / 100
        ];
    }

    public function getPaymentCapacity($blocks) {
        if(!isset($blocks['payment_capacity'])) {
            return null;
        }

        $block = $blocks['payment_capacity']->first();

        if(!$block || $block['sub_type'] == '0') {
            return [
                'description' => 'Capacidade de Pagamento',
                'message' => trim($block['message']),
                'value' => null
            ];
        }

        return [
            'description' => 'Capacidade de Pagamento',
            'value' => ((float) trim($block['capacity'])) / 100
        ];
    }

    public function getIncomeCommitment($blocks) {
        if(!isset($blocks['income_commitment'])) {
            return null;
        }

        $block = $blocks['income_commitment']->first();

        if(!$block || $block['sub_type'] == '0') {
            return [
                'description' => 'Comprometimento de Renda',
                'message' => trim($block['message']),
                'value' => null
            ];
        }

        return  [
            'description' => 'Comprometimento de Renda',
            'value' => ((float) $block['commit']) / 100
        ];
    }

    public function translate() {
        $blocks = $this->getResponseBlocks()->groupBy('type_rule');

        if(!isset($blocks['B001']) || !isset($blocks['B003']) || !isset($blocks['B004'])) {
            return null;
        }

        return [
            'idenification' => $this->getIdentification($blocks),
            'situation' => $this->getSituation($blocks),
            'address' => $this->getAddress($blocks),
            'contact' => $this->getContact($blocks),
            'summaries' => $this->getSummaries($blocks),
            'queries' => [
                'summary' => $this->getQueriesSummary($blocks),
                'details' => $this->getQueries($blocks),
            ], 
            'score' => $this->getScore($blocks),
            'protests' => $this->getProtests($blocks),
            'partnerParticipation' => $this->getPartnerParticipation($blocks),
            'additionalData' => $this->getAdditionalData($blocks),
            'syntheticRegistration' => $this->getSyntheticRegistration($blocks),
            'overdueDebt' => $this->getOverdueDebt($blocks),
            'badChecks' => $this->getBadChecks($blocks),
            'pendencies' => $this->getPendencies($blocks),
            'judicialActions' => $this->getJudicialActions($blocks),
            'bankruptcies' => $this->getBankruptcies($blocks),

        ];
    }

}